; ----------------------------------------------------------------------------
; Fonte numerica para LCD de 2 linhas e 16 colunas na familia PIC18F
;
; Esta fonte e' dividida em tres partes:
;
;  1 - "halfsz2x3f18eep_chars.inc" ... arquivo de caracteres a serem
;                                      definidos na CGRAM do LCD.
;
;  2 - "halfsz2x3f18eep_font.inc" .... arquivo de desenho dos caracteres
;                                      residentes na EEPROM.
;
;  3 - "halfsz2x3f18eep_src.inc" ..... arquivo de controle com funcoes
;                                      para exibicao de caracteres no LCD.
; ----------------------------------------------------------------------------



; ----------------------------------------------------------------------------
; Pre-requisitos                                                    02/09/2016
; ----------------------------------------------------------------------------

; 1 - Carga da Macro READ_EEPROM.mac
  ifndef __MACROREADEEPROM
    messg "Fonte halfsz2x3f18eep_src - READ_EEPROM.mac incluida"
    #include "Includes\READ_EEPROM.mac"
  endif




; ----------------------------------------------------------------------------


; ----------------------------------------------------------------------------
; MODO DE USAR:
;
;  1 - Incluir os seguintes registradores no programa:
;_rgcharct   res   .1
;_rglinha    res   .1
;_rgcoluna   res   .1
;_rgptrl     res   .1
;
;  2 - Incluir o arquivo "halfsz2x3f18eep_src.inc" no final do programa.
;      Esse arquivo fara os includes automaticos dos arquivos:
;        "Includes\READ_EEPROM.mac"
;        "Includes\halfsz2x3f18eep_chars.inc" 
;        "Includes\halfsz2x3f18eep_font.inc" 
;
; 3 - Apos a configuracao do LCD, carregar a CGRAM com os caracteres
;     especias. Exemplo:
;           call   font_load            ; carga da CGRAM
;
; 4 - Para enviar um caracter ao LCD. Exemplo:
;           movlw    .1                 ; numero da linha
;           movwf    _rgcoluna          ; especifica a coluna do LCD
;           movlw    0x08               ; caracter a enviar "8"
;           call     font_put           ; envia o caracter
;           movlw    0x01               ; caracter a enviar "1"
;           call     font_put           ; envia o caracter
; ----------------------------------------------------------------------------
; NOTAS:
;
; 1 - O numero da coluna sera' automaticamente incrementado para aproxima 
;     posicao de caracter no LCD.
;
; 2 - Nao e' necessario especificar o numero da linha porque esta fonte
;     e' para LCDs de 2 linhas apenas e as funcoes de exibicao se
;     encarregam de posiciona-las.
;
; ----------------------------------------------------------------------------

_charcell   equ   .3                    ; largura do caracter no LCD (colunas)
; ----------------------------------------------------------------------------
; FUNCAO: font_put
;
; Envia um caracter para o LCD
;
; Entrada: W contem o codigo do caracter - (0x00 a 0x0F)
;
; Saida  : Caracter enviado ao LCD.
;
; ----------------------------------------------------------------------------
font_put    movwf    _rgptrl            ; salva o codigo do caracter

            ;
            ; Calcula o endereco da EEPROM onde reside o caracter
            ;
            bcf      STATUS,C           ; Carry=0

            rlcf      _rgptrl, f         ; x2
            rlcf      _rgptrl, f         ; x4
            rlcf      _rgptrl, f         ; x8

            ;
            ; Mostra caracteres da primeira linha
            ;
            movlw    .1                 ; primeira linha
            movwf    _rglinha           ; estabelece a proxima linha
            SETI2C_CURSOR _rglinha, _rgcoluna, "r"
            movlw    _charcell          ; largura
            movwf    _rgcharct          ; carrega contador
            call     lcdOpen
font_put01  movf     _rgptrl, w         ; pega indexador         
            READ_EEPROM                 ; le EPPROM
            call     lcdSendW           ; envia codigo lido
            incf     _rgptrl, f         ; incrementa endereco na EEPROM
            decfsz   _rgcharct, f       ; conta colunas
            goto     font_put01         ; pega o proximo
            call     lcdClose
            incf     _rgptrl, f         ; pula o ultimo 0x00 em numxxx

            ;
            ; Mostra caracteres da segunda linha
            ;
            movlw    .2                 ; segunda linha
            movwf    _rglinha           ; estabelece a proxima linha
            SETI2C_CURSOR _rglinha, _rgcoluna, "r"
            movlw    _charcell          ; largura
            movwf    _rgcharct          ; carrega contador
            call     lcdOpen
font_put02  movf     _rgptrl, w         ; pega indexador         
            READ_EEPROM                 ; le EPPROM
            call     lcdSendW           ; envia codigo lido
            incf     _rgptrl, f         ; incrementa endereco na EEPROM
            decfsz   _rgcharct, f       ; conta colunas
            goto     font_put02         ; pega o proximo
            movlw    _charcell          ; largura
            addwf    _rgcoluna, f       ; avanca para proxima coluna
            call     lcdClose
            return

; ----------------------------------------------------------------------------
; FUNCAO: font_load
;
; Carrega a CGRAM do LCD com o grupo de caracteres contidos na tabela
; definida pela inclusao do arquivo "halfsz2x3f18eep_chars.inc".
; 
; Entrada: Nao ha
;
; Saida  : CGRAM do LCD carregada com o grupo de caracteres.
; ----------------------------------------------------------------------------
font_load 
         movlw    0x00                    ; endereco na CGRAM
         call     lcdSetCgram             ; estabelece o endereco
         movlw    .64                     ; numero de bytes
         movwf    R12                     ; carrega o contador
         movlw    upper(halfsz_charset)
         movwf    TBLPTRU
         movlw    high(halfsz_charset)
         movwf    TBLPTRH
         movlw    low(halfsz_charset)
         movwf    TBLPTRL
         call     lcdOpen
fonte_fill
         tblrd    *+                      ; le a tabela
         movf     TABLAT  , w             ; pega um caracter
         call     lcdSendW                ; envia para CGRAM do LCD
         decfsz   R12,f                   ; conta byte enviado
         goto     fonte_fill              ; envia o proximo
         call     lcdClose
         return                           ; fim 


   #include "halfsz2x3f18eep_chars.inc"
   #include "halfsz2x3f18eep_font.inc"
; Nao mudar a ordem dos includes acima.




